// Copyright (C) 2012 - Michael Baudin
// Copyright (C) 2009 - DIGITEO - Michael Baudin
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt

function demoLatticeUrand3D()
//
// Generate uniform numbers in [0,1]^2
// Keep only those in [0,t]^2,
// where t = 0.001.
// We stop when we have N such points.
// We proceed by blocks of NBlock points (X1,X2).
// The probability P((X1,X2) in [0,t])=t^2.
// Therefore, each trial produces 
// NBlock*t^2 new points.
// For example, with NBlock=1.e7 and t=0.001, this is 10 new points 
// for each experiment.
// This is slow...
// We see how "urand" produces points on a lattice.
//
stacksize("max");
sz=stacksize();
nfree = sz(1)-sz(2);
NBlock = floor(nfree/100);
RFull = [];
N = 1.e4;
t = 0.01;
mprintf("t = %e\n",t);
mprintf("Nblock = %d\n",NBlock);
mprintf("N = %d\n",N);
i = 1;
h = scf();
distfun_genset("urand");
    subplot(2,2,1);
    xtitle("RNG : ""urand""","X1","X2")
    subplot(2,2,2);
    xtitle("RNG : ""urand""","X1","X3")
    subplot(2,2,4);
    xtitle("RNG : ""urand""","X2","X3")
while (%t)
    mprintf("Block #%d\n",i);
    R = distfun_unifrnd(0,1,NBlock,3);
    k = find(R(:,1)<t & R(:,2)<t & R(:,3)<t);
	nbnew = size(k,"r");
    R = R(k,:);
    RFull = [RFull;R];
    nfull = size(RFull,"r");
    mprintf("  New = %d, Total=%d points in [0,t)^3\n",...
	    nbnew,nfull)
    if (nfull>N) then
        break
    end
    i = i+1;
	if ( R <> [] ) then
    drawlater();
    subplot(2,2,1);
    plot(R(:,1),R(:,2),"b.");
    subplot(2,2,2);
    plot(R(:,1),R(:,3),"b.");
    subplot(2,2,4);
    plot(R(:,2),R(:,3),"b.");
    h.children(1).children.children.mark_size = 1;
    h.children(2).children.children.mark_size = 1;
    h.children(3).children.children.mark_size = 1;
    drawnow();
	end
end

endfunction 
demoLatticeUrand3D();
clear demoLatticeUrand3D


//
// Load this script into the editor
//
filename = "lattice_urand3D.sce";
dname = get_absolute_file_path(filename);
editor ( fullfile(dname,filename) );
