
/*
 * Copyright (C) 2012 - Michael Baudin
 * Copyright (C) 2011 - DIGITEO - Michael Baudin
 * Copyright (C) 2008 - INRIA
 *
 * This file must be used under the terms of the CeCILL.
 * This source file is licensed as described in the file COPYING, which
 * you should have received as part of this distribution.  The terms
 * are also available at
 * http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
 *
 */
 
#ifndef _CDFLIB_H_
#define _CDFLIB_H_

#ifdef _MSC_VER
	#if LIBCDFLIB_EXPORTS 
		#define CDFLIB_IMPORTEXPORT __declspec (dllexport)
	#else
		#define CDFLIB_IMPORTEXPORT __declspec (dllimport)
	#endif
#else
	#define CDFLIB_IMPORTEXPORT
#endif

#undef __BEGIN_DECLS
#undef __END_DECLS
#ifdef __cplusplus
# define __BEGIN_DECLS extern "C" {
# define __END_DECLS }
#else
# define __BEGIN_DECLS /* empty */
# define __END_DECLS /* empty */
#endif

__BEGIN_DECLS

#define min(a,b) ((a) <= (b) ? (a) : (b))
#define max(a,b) ((a) >= (b) ? (a) : (b))
#define TRUE_ (1)
#define FALSE_ (0)


// From Scilab
extern /* Subroutine */ int basout_(int *, float *, char *, long int);
extern double dlamch_(char *, long int);

// If this function is non-NULL, it is used to print messages out.
// It can be configure with the public function "unifrng_msgsetfunction".
// If the function is not configured, the messages are printed in the console.
extern void (* cdflib_messagefunction)(char * message);

// Configure the function which prints messages.
// This function must be called before calling any 
// random number generation function.
void cdflib_messageSetFunction ( void (* f)(char * message) );

// Print error messages
void cdflib_messageprint(char * msg);

double pow_dd(double *ap, double *bp);
double d_sign(double *a, double *b);
double d_int(double *x);
double algdiv_(double *a, double *b);
double alnrel_(double *a);
double spmpar_(int *i__);
double snorm_(void);
int setgmn_(double *meanv, double *covm, int *
	ldcovm, int *p, double *parm, int *ierr);
int ipmpar_(int *i__);
double gennf_(double *dfn, double *dfd, double *xnonc);
double genf_(double *dfn, double *dfd);
int dinvr_0_(int n__, int *status, double *x, 
	double *fx, long int *qleft, long int *qhi, double *zsmall, 
	double *zbig, double *zabsst, double *zrelst, double *
	zstpmu, double *zabsto, double *zrelto);
int dinvr_(int *status, double *x, double *fx, 
	long int *qleft, long int *qhi);
int dstinv_(double *zsmall, double *zbig, double 
	*zabsst, double *zrelst, double *zstpmu, double *zabsto, 
	double *zrelto);
double rlog_(double *x);
double rlog1_(double *x);
double rexp_(double *x);
double rcomp_(double *a, double *x);
double psi1_(double *xx);
int ignnbn_(int *n, double *p);
double gsumln_(double *a, double *b);
int gratio_(double *a, double *x, double *ans, 
	double *qans, int *ind);
int grat1_(double *a, double *x, double *r__, 
	double *p, double *q, double *eps);
double genunf_(double *low, double *high);
int genprm_(double *array, int *larray);
double gennor_(double *av, double *sd);
double gennch_(double *df, double *xnonc);
int genmul_(int *n, double *p, int *ncat, 
	int *ix);
int genmn_(double *parm, double *x, double *work);
double gengam_(double *a, double *r__);
double genexp_(double *av);
double genchi_(double *df);
double gamma_(double *a);
double gamln_(double *a);
double gamln1_(double *a);
int gaminv_(double *a, double *x, double *x0, 
	double *p, double *q, int *ierr);
double gam1_(double *a);
double fpser_(double *a, double *b, double *x, double *
	eps);
double exparg_(int *l);
double esum_(int *mu, double *x);
double erfc1_(int *ind, double *x);
double erf_(double *x);
int dzror_0_(int n__, int *status, double *x, 
	double *fx, double *xlo, double *xhi, long int *qleft, 
	long int *qhi, double *zxlo, double *zxhi, double *zabstl, 
	double *zreltl);
int dzror_(int *status, double *x, double *fx, 
	double *xlo, double *xhi, long int *qleft, long int *qhi);
int dstzr_(double *zxlo, double *zxhi, double *
	zabstl, double *zreltl);
double dt1_(double *p, double *q, double *df);
double dinvnr_(double *p, double *q);
double dcddexpm1_(double *x);
double devlpl_(double *a, int *n, double *x);
int cumt_(double *t, double *df, double *cum, 
	double *ccum);
int cumpoi_(double *s, double *xlam, double *cum,
	 double *ccum);
int cumnor_(double *arg, double *result, double *
	ccum);
int cumnbn_(double *s, double *xn, double *pr, 
	double *ompr, double *cum, double *ccum);
int cumgam_(double *x, double *a, double *cum, 
	double *ccum);
int cumfnc_(double *f, double *dfn, double *dfd, 
	double *pnonc, double *cum, double *ccum);
int cumf_(double *f, double *dfn, double *dfd, 
	double *cum, double *ccum);
int cumchn_(double *x, double *df, double *pnonc,
	 double *cum, double *ccum);
int cumchi_(double *x, double *df, double *cum, 
	double *ccum);
int cumbin_(double *s, double *xn, double *pr, 
	double *ompr, double *cum, double *ccum);
int cumbet_(double *x, double *y, double *a, 
	double *b, double *cum, double *ccum);
int cdft_(int *which, double *p, double *q, 
	double *t, double *df, int *status, double *bound);
int cdfpoi_(int *which, double *p, double *q, 
	double *s, double *xlam, int *status, double *bound);
int cdfnor_(int *which, double *p, double *q, 
	double *x, double *mean, double *sd, int *status, 
	double *bound);
int cdfnbn_(int *which, double *p, double *q, 
	double *s, double *xn, double *pr, double *ompr, 
	int *status, double *bound);
int cdfgam_(int *which, double *p, double *q, 
	double *x, double *shape, double *scale, int *status, 
	double *bound);
int cdffnc_(int *which, double *p, double *q, 
	double *f, double *dfn, double *dfd, double *phonc, 
	int *status, double *bound);
int cdff_(int *which, double *p, double *q, 
	double *f, double *dfn, double *dfd, int *status, 
	double *bound);
int cdfchn_(int *which, double *p, double *q, 
	double *x, double *df, double *pnonc, int *status, 
	double *bound);
int cdfchi_(int *which, double *p, double *q, 
	double *x, double *df, int *status, double *bound);
int cdfbin_(int *which, double *p, double *q, 
	double *s, double *xn, double *pr, double *ompr, 
	int *status, double *bound);
int cdfbet_(int *which, double *p, double *q, 
	double *x, double *y, double *a, double *b, int *
	status, double *bound);
double bup_(double *a, double *b, double *x, double *y, 
	int *n, double *eps);
double brcomp_(double *a, double *b, double *x, double *y);
double brcmp1_(int *mu, double *a, double *b, double *x, 
	double *y);
int bratio_(double *a, double *b, double *x, 
	double *y, double *w, double *w1, int *ierr);
double bpser_(double *a, double *b, double *x, double *
	eps);
int bgrat_(double *a, double *b, double *x, 
	double *y, double *w, double *eps, int *ierr);
double bfrac_(double *a, double *b, double *x, double *y, 
	double *lambda, double *eps);
double betaln_(double *a0, double *b0);
double bcorr_(double *a0, double *b0);
double apser_(double *a, double *b, double *x, double *
	eps);
double alnrel_(double *a);
double alngam_(double *x);
double algdiv_(double *a, double *b);
double stvaln_(double *p);
double basym_(double *a, double *b, double *lambda, 
	double *eps);
int largestint(void);

// Parameters used in the cdf*.c functions

// Returns the relative tolerance.
// This is twice the machine epsilon.
double cdflib_rtol(void);

// Returns the absolute tolerance.
// This is the underflow threshold.
double cdflib_zero(void);

// This is the overflow threshold.
double cdflib_inf(void);

__END_DECLS

#endif /** _CDFLIB_H_   **/




