// Copyright (C) 2012 - Michael Baudin
// Copyright (C) 2009 - 2011 - DIGITEO - Michael Baudin
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt

// <-- JVM NOT MANDATORY -->
// <-- ENGLISH IMPOSED -->

//
// Check empty matrix
p = distfun_betacdf ( [] , [] , [] );
assert_checkequal ( p , [] );

//
// Check expansion of a and expansion of b
computed = distfun_betacdf ( 0.1:0.2:0.9 , 2 , 2 );
expected = [0.0280    0.2160    0.5000    0.7840    0.9720];
assert_checkalmostequal ( computed , expected , 10 * %eps );
//
// Check with expanded arguments
computed = distfun_betacdf ( 0.1:0.2:0.9 , [2 3 4 5 6], [5 4 3 2 1] );
expected = [0.114265    0.25569    0.34375    0.420175    0.531441];
assert_checkalmostequal ( computed , expected , 10 * %eps );
//
// Check bounds of x
computed = distfun_betacdf([0 1],2,3);
expected = [0 1];
assert_checkalmostequal ( computed , expected , 10 * %eps );
//
// Check bounds of x
instr = "computed = distfun_betacdf(-1,2,3)";
msg = "distfun_betacdf: Expected that all entries of input argument x at input #1 are in the range [0,1], but entry #1 is equal to -1.";
assert_checkerror(instr,msg);
//
// Check bounds of x
instr = "computed = distfun_betacdf(2,2,3)";
msg = "distfun_betacdf: Expected that all entries of input argument x at input #1 are in the range [0,1], but entry #1 is equal to 2.";
assert_checkerror(instr,msg);


/////////////////////////////////////////////////////////////////////
//
// Test accuracy of distfun_betacdf
//
path=distfun_getpath();
dataset = fullfile(path,"tests","unit_tests","beta","betacdf.dataset.csv");
table = assert_csvread ( dataset , "," , [] , "/#(.*)/" );
table = evstr(table);

precision = 100*%eps;

nt = size(table,"r");
for k = 1 : nt
  if ( k==9) then
    // TODO : fix this range of p
    continue
  end
  x = table(k,1);
  a = table(k,2);
  b = table(k,3);
  expected = table(k,4);
  computed = distfun_betacdf ( x , a , b );
  assert_checkalmostequal ( computed , expected , precision );
end

//
// Check vectorization
a = 1;
b = 3;
x = linspace(0.1,0.9,10);
p = distfun_betacdf ( x , a , b );
p2 = [];
for i = 1 : 10
    p2(1,i) = distfun_betacdf ( x(i) , a , b );
end
assert_checkequal(p,p2);

//
// Check upper tail
p = distfun_betacdf ( 0.1:0.2:0.9 , 2 , 2 );
q = distfun_betacdf ( 0.1:0.2:0.9 , 2 , 2 , %f );
assert_checkalmostequal ( p+q , ones(1,5) , 10 * %eps );
//
// Check upper tail with R
p = distfun_betacdf ( 0.5 , 1 , 2 );
assert_checkalmostequal ( p , 0.75 );
p = distfun_betacdf ( 0.5 , 1 , 2 , %f );
assert_checkalmostequal ( p , 0.25 );
p = distfun_betacdf ( 0.999 , 1 , 10 , %f );
assert_checkalmostequal ( p , 1.e-30 );
