// Copyright (C) 2012 - Michael Baudin
// Copyright (C) 2011 - INRIA - Michael Baudin
// Copyright (C) 2011 - DIGITEO - Michael Baudin
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt



// <-- JVM NOT MANDATORY -->
// <-- ENGLISH IMPOSED -->

//
// Check empty matrix
r = distfun_exprnd ( [] );
assert_checkequal ( r , [] );
//
// Set the seed so as to always get the same results.
distfun_seedset(1);

// Use R = distfun_exprnd ( mu )
c = distfun_exprnd(1:6);
e = [0.8746163    0.0056383    0.9841605    0.2792985    45.379874    12.328519];
assert_checkalmostequal(c,e,[],1.e-5);

// Check mean and variance for R = distfun_exprnd ( mu )
N = 1000;
for mu = 1:6;
  computed = distfun_exprnd(mu,[1 N]);
  c = mean(computed);
  e = mu;
  assert_checkalmostequal(c,e,1.e-1);
  c = st_deviation(computed);
  e = mu;
  assert_checkalmostequal(c,e,1.e-1);
end
//
// Check R = distfun_exprnd ( mu , v )
distfun_seedset(1);
c = distfun_exprnd(2,[1 5]);
e = [1.7492326    0.0056383    0.6561070    0.1396492    18.15195];
assert_checkalmostequal(c,e,[],1.e-5);
//
c = distfun_exprnd(2,[3 2]);
e = [
    4.1095065    3.8379693  
    2.3924553    2.8870931  
    0.0019199    4.7645861  
];
assert_checkalmostequal(c,e,[],1.e-5);
//
// Check mean and variance for R = distfun_exprnd ( a , b )
distfun_seedset(1);
N = 1000;
mu = 2;
computed = distfun_exprnd(mu,[1 N]);
c = mean(computed(1:N) );
e = mu;
assert_checkalmostequal(c,e,1.e-1);
c = st_deviation(computed(1:N) );
e = mu;
assert_checkalmostequal(c,e,1.e-1);
//
// Check R = distfun_exprnd ( mu , m , n )
distfun_seedset(1);
c = distfun_exprnd([1 2 3;4 5 6],2,3);
e = [
    0.8746163    0.6561070    27.227925  
    0.0112766    0.3491231    12.328519  
];
assert_checkalmostequal(c,e,[],1.e-5);
//
c = distfun_exprnd(2,2,3);
e = [
    2.3924553    3.8379693    4.7645861  
    0.0019199    2.8870931    1.8497513  
];
assert_checkalmostequal(c,e,[],1.e-5);
//
c = distfun_exprnd(1,2,3);
e = [
    1.6806106    1.0625869    0.9244049  
    0.9469800    0.4008567    0.0666324  
];
assert_checkalmostequal(c,e,[],1.e-5);
  //
// Check mean and variance for R = distfun_exprnd ( mu )
N = 1000;
mu = 2;
computed = distfun_exprnd(mu,1,N);
c = mean(computed(1:N) );
e = mu;
assert_checkalmostequal(c,e,1.e-1);
c = st_deviation(computed(1:N) );
e = mu;
assert_checkalmostequal(c,e,1.e-1);
if (%f) then
    //
    // Make a plot of the actual distribution of the numbers
    mu = 2;
    data = distfun_exprnd(mu,1,1000);
    histplot(10,data)
    x = linspace(0,14,1000);
    y = distfun_exppdf(x,mu);
    plot(x,y);
end
