// Copyright (C) 2012 - Michael Baudin
// Copyright (C) 2009 - 2011 - DIGITEO - Michael Baudin
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt

// <-- JVM NOT MANDATORY -->
// <-- ENGLISH IMPOSED -->


//
// Check empty matrix
x = distfun_norminv ( [] , [] , [] );
assert_checkequal ( x , [] );
//
// Test distfun_norminv
//
computed = distfun_norminv ( [0.5 0.9 0.7] );
expected = [ 0.000000000000000   1.281551565544601   0.524400512708041 ];
assert_checkalmostequal ( computed , expected , 10 * %eps );
//
// Test argument expansion
computed = distfun_norminv ( [0.5 0.9 0.7] , 1.0 , 2.0 );
expected = [ 1.00000000000000   3.56310313108920   2.04880102541608 ];
assert_checkalmostequal ( computed , expected , 10 * %eps );
//
// Test with expanded arguments
computed = distfun_norminv ( [0.5 0.9 0.7] , [1 1 1] , [2 2 2] );
expected = [ 1.00000000000000   3.56310313108920   2.04880102541608 ];
assert_checkalmostequal ( computed , expected , 10 * %eps );

///////////////////////////////////////////////////////////////
//
// Test accuracy of distfun_norminv
//

precision = 100*%eps;
path=distfun_getpath();
dataset = fullfile(path,"tests","unit_tests","normal","norminv.dataset.csv");
table = assert_csvread ( dataset , "," , [] , "/#(.*)/" );
table = evstr(table);

nt = size(table,"r");
for k = 1 : nt
  p = table(k,1);
  mu = table(k,2);
  sigma = table(k,3);
  expected = table(k,4);
  computed = distfun_norminv ( p , mu , sigma );
  assert_checkalmostequal ( computed , expected , precision );
end

///////////////////////////////////////////////////////////////
//
// Test accuracy of distfun_norminv
//

path=distfun_getpath();
dataset = fullfile(path,"tests","unit_tests","normal","normal.yalta.dataset.csv");
table = assert_csvread ( dataset , "," , [] , "/#(.*)/" );
table = evstr(table);

mu = 0.;
sigma = 1.;
precision = 1.e-5;

nt = size(table,"r");
for k = 1 : nt
  p = table(k,1);
  expected = table(k,2);
  computed = distfun_norminv ( p , mu , sigma );
  assert_checkalmostequal ( computed , expected , precision );
end

///////////////////////////////////////////////////////////////
//
// Test accuracy of distfun_norminv

path=distfun_getpath();
dataset = fullfile(path,"tests","unit_tests","normal","normal.R.dataset.csv");
table = assert_csvread ( dataset , "," , [] , "/#(.*)/" );
table = evstr(table);

precision = 100*%eps;

ntests = size(table,"r");
for i = 1 : ntests
  expected = table(i,1);
  mu = table(i,2);
  sigma = table(i,3);
  p = table(i,5);
  computed = distfun_norminv ( p , mu , sigma );
  assert_checkalmostequal ( computed , expected , precision );
end

//
// Check vectorization
mu = 300;
sigma = 30;
p = linspace(0.1,0.9,10);
xx = distfun_norminv ( p , mu , sigma );
xx2 = [];
for i = 1 : 10
    xx2(1,i) = distfun_norminv ( p(i) , mu , sigma );
end
assert_checkequal(xx,xx2);
//
// See upper tail: compare with R
x=distfun_norminv(0.0001,1.,2.);
assert_checkalmostequal ( x , -6.4380329709113599534 , 1.e-15 );
x=distfun_norminv(0.0001,1.,2.,%f);
assert_checkalmostequal ( x ,  8.4380329709113599534 , 1.e-6 );
x=distfun_norminv(1.e-10,1.,2.,%f);
assert_checkalmostequal ( x , 13.722681804808111394 , 1.e-15 );

