// Copyright (C) 2012 - Michael Baudin
// Copyright (C) 2009 - 2011 - DIGITEO - Michael Baudin
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt

// <-- JVM NOT MANDATORY -->


//
// Check empty matrix
p = distfun_normpdf ( [] , [] , [] );
assert_checkequal ( p , [] );
//
// Check with default mu, sigma
computed = distfun_normpdf ( [-1 1] );
expected = [ 0.241970724519143   0.241970724519143 ];
assert_checkalmostequal ( computed , expected , 10 * %eps );
//
// Check expansion of mu and expansion of sigma
computed = distfun_normpdf ( [1 2 3] , 1.0 , 2.0 );
expected = [ 0.199471140200716   0.176032663382150   0.120985362259572 ];
assert_checkalmostequal ( computed , expected , 100 * %eps );
//
// Check with expanded arguments
computed = distfun_normpdf ( [1 2 3] , [1 1 1], [2 2 2] );
expected = [ 0.199471140200716   0.176032663382150   0.120985362259572 ];
assert_checkalmostequal ( computed , expected , 100 * %eps );



///////////////////////////////////////////////////////////////
//
// Test accuracy of distfun_normpdf

// [x shape scale P precision]

precision = 100*%eps;
path=distfun_getpath();
dataset = fullfile(path,"tests","unit_tests","normal","normpdf.dataset.csv");
table = assert_csvread ( dataset , "," , [] , "/#(.*)/" );
table = evstr(table);

ntests = size(table,"r");
for i = 1 : ntests
  x = table(i,1);
  mu = table(i,2);
  sigma = table(i,3);
  expected = table(i,4);
  computed = distfun_normpdf ( x , mu , sigma );
  assert_checkalmostequal ( computed , expected , precision );
  if ( %f ) then
    d = assert_computedigits ( computed , expected );
    mprintf("Test #%d/%d: Digits = %.1f\n",i,ntests,d);
  end
end

///////////////////////////////////////////////////////////////
//
// Test accuracy of distfun_normpdf

path=distfun_getpath();
dataset = fullfile(path,"tests","unit_tests","normal","normal.R.dataset.csv");
table = assert_csvread ( dataset , "," , [] , "/#(.*)/" );
table = evstr(table);

precision = 100*%eps;

ntests = size(table,"r");
for i = 1 : ntests
  x = table(i,1);
  mu = table(i,2);
  sigma = table(i,3);
  expected = table(i,4);
  computed = distfun_normpdf ( x , mu , sigma );
  assert_checkalmostequal ( computed , expected , precision );
  if ( %f ) then
    d = assert_computedigits ( computed , expected );
    mprintf("Test #%d/%d: Digits = %.1f\n",i,ntests,d);
  end
end

//
// Check vectorization
mu = 300;
sigma = 30;
x = linspace(mu-3*sigma,mu+3*sigma,10);
p = distfun_normpdf ( x , mu , sigma );
p2 = [];
for i = 1 : 10
    p2(1,i) = distfun_normpdf ( x(i) , mu , sigma );
end
assert_checkequal(p,p2);
//
// Check consistency CDF/PDF
n = 10;
mu = 2;
sigma = 4;
p = linspace(0.01,0.99,n)';
x = distfun_norminv(p,mu , sigma);
p1 = distfun_normpdf ( x , mu , sigma );
p2 = [];
for i = 1 : n
    p2(i) = derivative(list(distfun_normcdf,mu , sigma),x(i));
end
assert_checkalmostequal ( p1 , p2 , 1.e-7 , [] , "element");
